/******************************************
Program : 32kSRAM Example for Micro128
Company : Micromint, Inc.
*********************************************/
     
#include <mega128.h>
#include <MMRS485.h>                   // Micromints Library for using both USARTs
#include <stdio.h>                     // Standard I/O library    
#include <delay.h>  
// Macros to use in order to set individual bits in extended I/O space located in SRAM between 
// 0x60 and 0xFF can not use the CBI or SBI assembly instruction to clear or set a bit.    
#define SETBIT(port,bit)(port)|=1<<(bit)
#define CLRBIT(port,bit)(port)&=~(1<<(bit))
#define OFFSET 0x2000
// Declare your global variables here
int COM;                               // if COM = 0 then use USART0 if it = 1 then use USART1     
int CH0 @0x7ffe;

unsigned int Data @0xFFE;              // This the memory location where the 12-bit ADC conversion 
                                       //count is stored after calling the utility to read a channel

// Routines used to call the utilities for the 12-bit ADC
void(*SingleADC1)(void)=0x0FC02;  
void(*SingleADC0)(void)=0x0FC09;  
void RD_RAM(unsigned int);
void WR_RAM(unsigned int);
void WRRAM_0000h(unsigned int, unsigned char);
void RDRAM_0000h(unsigned int);
void main(void)
{
// Declare your local variables here

// External SRAM page configuration: 
//              -              / 0000h - 7FFFh
// Lower page wait state(s): None
// Upper page wait state(s): None
MCUCR=0x80;
XMCRA=0x00;    
CLRBIT(XMCRB,0);  //Make XMM0 bit 0 of the XMCRB register low enabling all of external SRAM
CLRBIT(XMCRB,1);  //Make XMM0 bit 1 of the XMCRB register low
CLRBIT(XMCRB,2);  //Make XMM0 bit 2 of the XMCRB register low

// Set up USART1's Baud rate at 9600 bps with a 11.0592 MHz Crystal
UCSR1A=0x00;      // RX EN, TX EN
UCSR1B=0x18;     // RX EN, TX EN
UCSR1C=0x06;     // 8N1
UBRR1H=0x00;    // Baud rate high - 9600
UBRR1L=0x47;   // Baud rate low 

COM = 1;                           // Use USART1  
DDRD.6 = 0;			   // Make PORTD.6 an output  
PORTD.6 = 1; 			   // Enable the RS485 control line  

while (1)
      {     
      (*SingleADC0)();
      CH0 = Data;
      printf("CH0 = %d\r\n",CH0); 
      printf("\r\n");
      WR_RAM(0x2000);
      WRRAM_0000h(0x0000, 39);
      WRRAM_0000h(0x0001, 40);
      WRRAM_0000h(0x0002, 41);
      RDRAM_0000h(0x0000);
      RDRAM_0000h(0x0001);
      RDRAM_0000h(0x0002);
      RD_RAM(0x2000); 
      delay_ms(3000); 
      };
}                       

void WR_RAM(unsigned int MemAddr)
{
   int *p;
   p=MemAddr;    
   while(p < (MemAddr + 10))
   {     
     (*SingleADC1)();
     *p = Data;  
     printf("Wrote %d", Data);
     printf(" to location %X", p);
     printf("    \r\n");         
     p++;
   } 
   printf("\r\n");
}                 
   
void RD_RAM(unsigned int MemAddr)
{
   int CH1;
   int *p;
   p=MemAddr;
   while(p < (MemAddr + 10))
   {               
     CH1=*p; 
     printf("Read %d", CH1);
     printf(" at location %X", p);
     printf("    \r\n");    
     p++;
   }       
   printf("\r\n");
}

void WRRAM_0000h(unsigned int MemAddr, unsigned char WRData)
{
   unsigned char *p = (unsigned char *) (OFFSET + 1);   // This assignment is called casting.
   p = OFFSET + MemAddr;  
   
   DDRC = 0xFF;   // Set the released port pin PORTC.7, PORTC.6, and PORTC.5 to outputs
   PORTC = 0x00;  // Make the outputs low
   // Settng the following two bits releases some of PORTC from the External Memory Access
   SETBIT(XMCRB,0);  //Make XMM0 bit 0 and bit 1 of the XMCRB register high to make external
   SETBIT(XMCRB,1);  //SRAM enabled up to address 1FFFH. 
   *p = WRData; 
    printf("Wrote %d", WRData);
    printf(" to location %X", p-OFFSET);
    printf("    \r\n");         
    printf("\r\n"); 
   // Clear the following two bits to make all of PORTC used for External Memory Access
   CLRBIT(XMCRB,0);  //Make XMM0 bit 0 of the XMCRB register low enabling all of external SRAM
   CLRBIT(XMCRB,1);  //Make XMM0 bit 1 of the XMCRB register low
}                 
   
void RDRAM_0000h(unsigned int MemAddr)
{
  unsigned char *p = (unsigned char *) (OFFSET + 1);   // This assignment is called casting.
  unsigned char RDData;   
    p = OFFSET + MemAddr; 
    DDRC = 0xFF;   // Set the released port pin PORTC.7, PORTC.6, and PORTC.5 to outputs
    PORTC = 0x00;  // Make the outputs low
    // Settng the following two bits releases some of PORTC from the External Memory Access
    SETBIT(XMCRB,0);  //Make XMM0 bit 0 and bit 1 of the XMCRB register high to make external
    SETBIT(XMCRB,1);  //SRAM enabled up to address 1FFFH. 
    RDData = *p; 
    printf("Read %d", RDData);
    printf(" to location %X", p-OFFSET);
    printf("    \r\n");         
    printf("\r\n"); 
   // Clear the following two bits to make all of PORTC used for External Memory Access
    CLRBIT(XMCRB,0);  //Make XMM0 bit 0 of the XMCRB register low enabling all of external SRAM
    CLRBIT(XMCRB,1);  //Make XMM0 bit 1 of the XMCRB register low
}